PetBattleTeams = CreateFrame("frame")
PetBattleTeams.START = 1
PetBattleTeams.FINISHED = 5


--tooltip functions

--Creates fake of blizzards PetBattleUnitTooltip
--Returns a tooltip
PetBattleTeams.CreateTooltip = function(self)
	tooltip = CreateFrame("frame","PetBattleTeamTooltip",nil,"PetBattleUnitTooltipTemplate")
	tooltip:SetHeight(211)
	tooltip.WeakToLabel:Hide()
	tooltip.ResistantToLabel:Hide()
	tooltip.rarityGlow = tooltip:CreateTexture("PetBattleTeamTooltipGlow","OVERLAY")
	tooltip.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	tooltip.rarityGlow:SetBlendMode("ADD")
	tooltip.rarityGlow:ClearAllPoints()
	tooltip.rarityGlow:SetDrawLayer("OVERLAY", 0)
	tooltip.rarityGlow:SetWidth(tooltip.Icon:GetWidth() * 1.7)
	tooltip.rarityGlow:SetHeight(tooltip.Icon:GetHeight() * 1.7)
	tooltip.rarityGlow:SetPoint("CENTER", tooltip.Icon, "CENTER", 0, 0)
	
	return tooltip
end

--Sets the tooltip to the currently moused over unit
PetBattleTeams.ToolTipSetUnit = function(self,tooltip, petID, team, petIndex)
	local speciesID, customName, level, xp, maxXp, displayID, petName, petIcon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID)
	local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID)
	local r, g, b,hex = GetItemQualityColor(rarity-1)
	local pet = PetBattleTeamsSettings.teams[team][petIndex]
	
	tooltip.rarityGlow:SetVertexColor(r, g, b)
	tooltip.Icon:SetTexture(petIcon)
	tooltip.Name:SetText("|c"..hex..petName.."|r")
	tooltip.Level:SetText(level)
	tooltip.XPBar:SetWidth(max((xp / max(maxXp,1)) * tooltip.xpBarWidth, 1))
	tooltip.Delimiter:SetPoint("TOP", tooltip.XPBG, "BOTTOM", 0, -10)
	tooltip.XPBar:Show()
	tooltip.XPBG:Show()
	tooltip.XPBorder:Show()
	tooltip.XPText:SetFormattedText(tooltip.xpTextFormat or PET_BATTLE_CURRENT_XP_FORMAT, xp, maxXp)
	tooltip.XPText:Show()
	tooltip.AttackAmount:SetText(attack)
	tooltip.SpeedAmount:SetText(speed)
	tooltip.PetType.Icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
	tooltip.AbilitiesLabel:Show()
	
	if customName then
		if ( customName ~= petName ) then
			tooltip.Name:SetText(customName)
			tooltip.SpeciesName:SetText(petName)
			tooltip.SpeciesName:Show()
		end
	else
		tooltip.SpeciesName:Hide()
	end
	
	if ( tooltip.HealthText ) then
		tooltip.HealthText:SetFormattedText(tooltip.healthTextFormat or PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth)
	end
	
	if ( health == 0 ) then
		tooltip.ActualHealthBar:SetWidth(1)
		else
		tooltip.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * tooltip.healthBarWidth)
	end
		
	for i=1, #pet.abilities do
		local name, icon, petType = C_PetJournal.GetPetAbilityInfo(pet.abilities[i])
		local abilityIcon = tooltip["AbilityIcon"..i]
		local abilityName = tooltip["AbilityName"..i]
		
		if ( name ) then
			abilityIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Neutral")
			abilityName:SetText(name)
			abilityName:Show()
		else
			abilityName:Hide()
			abilityIcon:Hide()
		end
	end

end


--Add team button related functions
PetBattleTeams.CreateAddTeamButton = function(self,parentName)
	local button = CreateFrame("BUTTON", parentName,PetJournalParent)
	IconTexture = button:CreateTexture(parentName.."Icon","ARTWORK")
	button:SetSize(40,40)
	IconTexture:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
	IconTexture:SetAllPoints(IconTexture:GetParent())
	
	button:SetScript("OnClick",function(self) 
		PetBattleTeams:CreateNewTeam()
		PetBattleTeams:CreatePetUnitFrames()
	end)
	return button
end

PetBattleTeams.UpdateAddTeamButton = function(self)
	if self.addTeamButton then
		if	#PetBattleTeamsSettings.teams < self.maxTeams and #PetBattleTeamsSettings.teams > 0 then
			self.addTeamButton:SetPoint("TOPLEFT","PetTeam"..#PetBattleTeamsSettings.teams.."Button"..1,"BOTTOMLEFT",-2,0)
			self.addTeamButton:Show();
		else
			self.addTeamButton:Hide();
		end
	end
	
end

--pet unitframe functions
PetBattleTeams.CreatePetButton = function(self,team,petIndex)
	
	local parentName = "PetTeam"..team.."Button"..petIndex
	local button = CreateFrame("BUTTON", parentName, PetJournalParent, "PetBattleMiniUnitFrameAlly")

	--fix template sizes
	button:SetSize(40,40)
	button.healthBarWidth = 37
	button.BorderAlive:SetSize(50,50)
	button.BorderDead:SetSize(50,50)
	
	if not button.rarityGlow then button.rarityGlow = button:CreateTexture(parentName.."Glow","ARTWORK") end

	button.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	button.rarityGlow:SetBlendMode("ADD")
	button.rarityGlow:ClearAllPoints()
	button.rarityGlow:SetDrawLayer("ARTWORK", 0)
	button.rarityGlow:SetWidth(button:GetWidth() * 1.6)
	button.rarityGlow:SetHeight(button:GetHeight() * 1.4)
	button.rarityGlow:SetPoint("CENTER", button.Icon, "CENTER", 0, 4)
	
	--[[button.levelBubble = button:CreateTexture(parentName.."LevelBubble","OVERLAY")
	button.levelBubble:SetTexture("Interface\\PetBattles\\PetJournal")
	button.levelBubble:SetSize(21,21)
	button.levelBubble:SetTexCoord(0.06835938,0.10937500,0.02246094,0.04296875)
	button.levelBubble:SetPoint("CENTER",button,"TOPLEFT",8,-8)
	
	button.level = button:CreateFontString(parentName.."Level","OVERLAY","GameFontNormalSmall")
	button.level:SetJustifyH("CENTER")
	button.level:SetText("XX")
	button.level:SetSize(0,0)
	button.level:SetPoint("CENTER",button.levelBubble,"CENTER",0,0)--]]
	
	if not button.selected then button.selected = button:CreateTexture(parentName.."Selected","OVERLAY") end
	button.selected:SetSize(28,28)
	button.selected:SetTexture("Interface\\PetBattles\\PetJournal")
	button.selected:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	button.selected:SetPoint("CENTER",button,"TOPRIGHT",0,0)
	button.selected:Hide()
	
	button:SetScript("OnLoad",nil)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnClick", function(self,button) 
	
		
		if ( button == "RightButton" ) then
			if #PetBattleTeamsSettings.teams > 1 then
				StaticPopup_Show("PBT_TEAM_DELETE",self.team,nil,self.team)
			end
		else
			
			PetBattleTeams:SetTeam(self.team)
		end
	
	end)

	button:SetScript("OnEnter",function(self) 
		PetBattleUnitTooltip_Attach(PetBattleTeams.tooltip, "TOPLEFT", self, "BOTTOMRIGHT", 0, 0)
		PetBattleTeams:ToolTipSetUnit(PetBattleTeams.tooltip, self.petID, self.team, self.petIndex)
		PetBattleTeams.tooltip:Show()
	end)
	button:SetScript("OnLeave",function(self)
		if ( PetBattleTeams.tooltip:GetParent() == self ) then
			PetBattleTeams.tooltip:Hide()
		end
	end)
	button:UnregisterAllEvents()
	
	--[[local IconTexture = button:CreateTexture(parentName.."IconTexture","ARTWORK")
	IconTexture:SetTexture(icon)
	--IconTexture:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
	IconTexture:SetAllPoints(IconTexture:GetParent())
	
	
	local NormalTexture = button:CreateTexture(parentName.."NormalTexture")
	NormalTexture:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	NormalTexture:SetSize(115,115)
	NormalTexture:SetPoint("CENTER",0,-1)
	button:SetNormalTexture(NormalTexture)
	
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	
	local CheckedTexture = button:CreateTexture(parentName.."CheckedTexture")
	CheckedTexture:SetTexture("Interface\\Buttons\\CheckButtonHilight")
	CheckedTexture:SetAllPoints(CheckedTexture:GetParent())
	CheckedTexture:SetBlendMode("ADD")
	button:SetCheckedTexture(CheckedTexture)--]]
	--PrintObject(button)
	
	return button
end

PetBattleTeams.CreatePetUnitFrames = function(self)
	local unitFrames = {}
	for team=1,self.maxTeams do 
		local buttons = {}
		for petIndex =1, 3 do 
			button = PetBattleTeams:CreatePetButton(team,petIndex)
			
			if petIndex==1 and team == 1 then
				button:SetPoint("TOPLEFT",PetJournalParent,"TOPRIGHT",3,0)
			elseif petIndex == 1 and team > 1 then
				button:SetPoint("TOPLEFT","PetTeam"..(team-1).."Button"..petIndex,"BOTTOMLEFT",0,-4)
			else
				button:SetPoint("LEFT","PetTeam"..team.."Button"..petIndex-1,"RIGHT",5,0)
			end
			
			table.insert(buttons,button);
			
		end
		table.insert(unitFrames,buttons)
	end
	
	return unitFrames;
end

PetBattleTeams.UpdateAllTeams = function(self)
	for i=1, self.maxTeams do
		self:UpdatePetTeam(i)
	end
	self:UpdateAddTeamButton()
end

PetBattleTeams.UpdatePetTeam = function(self,team)
	for i=1,3 do
		self:UpdatePetButton(team,i);
		--if _G["PetTeam"..team.."Button"..i] then  self.UpdatePetButton(_G["PetTeam"..team.."Button"..i]) end
	end
end

PetBattleTeams.UpdatePetButton=function(self,team,petIndex)
	if team < 1 or team > 14 or petIndex < 1 or petIndex > 3 then return end;
	
	button = self.petUnitFrames[team][petIndex]
	
	button.team = team
	button.petIndex = petIndex
	button:Hide()
	
	if PetBattleTeamsSettings.teams[button.team] and PetBattleTeamsSettings.teams[button.team][button.petIndex] then
		
		button.petID = PetBattleTeamsSettings.teams[button.team][button.petIndex].petID
		if button.petID and button.petID > 0 and C_PetJournal.GetPetInfoByPetID(button.petID) then

			-- get info about the pet
			local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(button.petID)
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(button.petID)
			
			button.selected:Hide()
			-- Is the right most pet and the selected team
			if (not PetBattleTeamsSettings.teams[button.team][button.petIndex+1] or ( PetBattleTeamsSettings.teams[button.team][button.petIndex+1] and  not C_PetJournal.GetPetInfoByPetID(PetBattleTeamsSettings.teams[button.team][button.petIndex+1].petID))) and PetBattleTeamsSettings.selected == button.team then
				button.selected:Show()
			end
			
			local r, g, b = GetItemQualityColor(rarity-1)
			button.rarityGlow:SetVertexColor(r, g, b)
			
			--button.level:SetText(level)
			
			button.Icon:SetTexture(icon)

			--set health bar
			if ( health == 0 ) then
				button.ActualHealthBar:Hide()
			else
				button.ActualHealthBar:Show()
			end
			button.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * button.healthBarWidth)
			
			--Set alive border
			if ( health == 0 ) then
				button.BorderAlive:Hide()
				button.rarityGlow:Hide()
			else
				button.BorderAlive:Show()
				button.rarityGlow:Show()
			end
			
			--set dead border
			if ( health == 0 ) then
				button.BorderDead:Show()
				button.rarityGlow:Hide()
			else
				button.BorderDead:Hide()
				button.rarityGlow:Show()
			end
			
			for _, object in pairs(button.hideWhenDeadList) do
				if ( health == 0 ) then
					object:Hide()
				else
					object:Show()
				end
			end
			
			button:Show()
		end
	end
end


--high level team functions
PetBattleTeams.RemoveTeam = function(self,team)
	table.remove(PetBattleTeamsSettings.teams,team)
	if #PetBattleTeamsSettings.teams > 0 then
		PetBattleTeams:SetTeam(1)
	end

	if #PetBattleTeamsSettings.teams < 14 then
			self.addTeamButton:Show()
	end
	PetBattleTeams:UpdateAllTeams()
end

PetBattleTeams.CreateNewTeam = function(self) 
	if #PetBattleTeamsSettings.teams < self.maxTeams then
		local team = {}
		for i = 1,3 do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			if pet.petID and pet.petID > 0 then
				table.insert(team,pet)
			end
		end
		
		table.insert(PetBattleTeamsSettings.teams,team)
		PetBattleTeamsSettings.selected = #PetBattleTeamsSettings.teams
		if #PetBattleTeamsSettings.teams == 14 then
				self.addTeamButton:Hide()
		end
		
		PetBattleTeams:UpdateAllTeams()
	end
end

PetBattleTeams.RepairTeams=function(self)
	for tx=1,2 do 
		for k,v in pairs(PetBattleTeamsSettings.teams) do
			for x,y in ipairs(v) do
				if not C_PetJournal.GetPetInfoByPetID(y.petID) then table.remove(v,x) end
				if #v == 0 then
					table.remove(PetBattleTeamsSettings.teams,k)
				end
			end
		end
	end
	self:UpdateAllTeams()
end


PetBattleTeams.UpdateCurrentTeam =function()
	if not PetBattleTeams:IsWorking() then
		local team = {}
		for i = 1,3 do
			pet = {}
			pet.abilities = {}
			pet.petID, pet.abilities[1], pet.abilities[2], pet.abilities[3] = C_PetJournal.GetPetLoadOutInfo(i)
			if pet.petID and pet.petID > 0 then
				table.insert(team,pet)
			end
		end
		
		PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] = team
		PetBattleTeams:UpdateAllTeams()
	end
end

PetBattleTeams.SetTeam = function(self,team)
	oldTeam = PetBattleTeamsSettings.selected
	PetBattleTeamsSettings.selected = team
	
	PetBattleTeams:UpdatePetTeam(team)
	PetBattleTeams:UpdatePetTeam(oldTeam)
	PetBattleTeams:StartWork()
end


--Pet switching logic
PetBattleTeams.OnUpdate = function(self)
	if self.step < 4 then    
		if PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected] and PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step] and C_PetJournal.GetPetInfoByPetID(PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step].petID) then
			pet = PetBattleTeamsSettings.teams[PetBattleTeamsSettings.selected][self.step]			
			local petID, ability1, ability2, ability3 = C_PetJournal.GetPetLoadOutInfo(self.step)
			if petID ~= pet.petID then 
				C_PetJournal.SetPetLoadOutInfo(self.step,pet.petID) 
				return
			else
				if ability1 ~= pet.abilities[1] then 
					C_PetJournal.SetAbility(self.step, 1, pet.abilities[1])
					return
				elseif ability2 ~= pet.abilities[2] then
					C_PetJournal.SetAbility(self.step, 2, pet.abilities[2])
					return
				elseif ability3 ~= pet.abilities[3] then
					C_PetJournal.SetAbility(self.step, 3, pet.abilities[3])
					return
				end
			end
			
			if petID == pet.petID and ability1 == pet.abilities[1] and ability2 == pet.abilities[2] and ability3 == pet.abilities[3] then
				self.step = self.step +1
			end
		else --invalid pet / non existant
			C_PetJournal.SetPetLoadOutInfo(self.step,0)
			if C_PetJournal.GetPetLoadOutInfo(self.step) == 0 then
				self.step = self.step +1
			end
		end 
	else
		PetJournal_UpdatePetLoadOut()
		self:SetScript("OnUpdate",nil)
		self.step = self.FINISHED
	end 
end

PetBattleTeams.StartWork = function(self)
	self.step = self.START
	self:SetScript("OnUpdate",PetBattleTeams.OnUpdate)
end

PetBattleTeams.IsWorking = function(self)
	return self.step < self.FINISHED 
end

--Reset saved variables
PetBattleTeams.Reset = function(self)
	print("PetBattleTeamsSettings Reset")
	PetBattleTeamsSettings={}
	PetBattleTeamsSettings.version = PetBattleTeams.version
	PetBattleTeamsSettings.teams = {}
	PetBattleTeamsSettings.selected = 1
	PetBattleTeams:CreateNewTeam()
end


-- misc functions
PetBattleTeams.OnEvent = function(self,event,...)
    if event == "ADDON_LOADED" then
        name = ...
        if name == "Blizzard_PetJournal" then 
			self:UnregisterEvent("ADDON_LOADED")
		elseif name == "PetBattleTeams" then
			local loaded, reason = LoadAddOn("Blizzard_PetJournal")
			if not PetBattleTeamsSettings or not PetBattleTeamsSettings.selected or not PetBattleTeamsSettings.teams or not PetBattleTeamsSettings.version or PetBattleTeamsSettings.version ~= PetBattleTeams.version then
				self:Reset()
			end
			if loaded then 
				self:UnregisterEvent("ADDON_LOADED")
				self.tooltip = self:CreateTooltip()
				self.petUnitFrames = self:CreatePetUnitFrames()
				self.addTeamButton = self:CreateAddTeamButton("AddPetTeamButton");
				self:RepairTeams();
				self:UpdateAllTeams()
			end
			
        end
    end
	if event == "PET_JOURNAL_LIST_UPDATE" then 
		self:UpdateAllTeams()
	end
	if event =="PET_JOURNAL_PET_DELETED" then	
		petID = ...
		self:RepairTeams(petID)
	end
end

StaticPopupDialogs["PBT_TEAM_DELETE"] = {
	text = "Are you sure you want to delete team |cffffd200%s|r?",
	button1 = OKAY,
	button2 = CANCEL,
	OnAccept = function(self)
		PetBattleTeams:RemoveTeam(self.data)
		PetBattleTeams:UpdateAllTeams()
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1,
}

function SlashCmdList.PETBATTLETEAMS(msg, editbox)
	id = tonumber(msg)
	if id and id > 0 and id <= #PetBattleTeamsSettings.teams then
		PetBattleTeams:SetTeam(id)
	elseif msg == "reset" then
		PetBattleTeams:Reset()
	else
		print("To switch between pet teams use /pbt # or /petbattleteams #, where # is a number between 1 and ".. #PetBattleTeamsSettings.teams)
		print("To reset your PBT setting use /pbt reset")
	end
end



function PrintObject(object,prepend)
	if not prepend then prepend = "   " end
	
	if object and type(object) == "table" then
		for k,v in pairs(object) do 
			if type(v) == "table" then 
				print(prepend..k) 
				PrintObject(v,prepend.."  ")
			else
				print (prepend..k.." : "..tostring(v))
			end
			
		end
	end
end


--constants
SLASH_PETBATTLETEAMS1, SLASH_PETBATTLETEAMS2 = '/pbt', '/petbattleteams' 

PetBattleTeams.version = 7
PetBattleTeams.maxTeams = 14
PetBattleTeams.step = 5

--Begin addon
PetBattleTeams:RegisterEvent("PET_JOURNAL_PET_DELETED")
PetBattleTeams:RegisterEvent("PET_JOURNAL_LIST_UPDATE")
PetBattleTeams:RegisterEvent("ADDON_LOADED")
PetBattleTeams:SetScript("OnEvent" , PetBattleTeams.OnEvent)

hooksecurefunc(C_PetJournal, "SetPetLoadOutInfo", PetBattleTeams.UpdateCurrentTeam)
hooksecurefunc(C_PetJournal, "SetAbility", PetBattleTeams.UpdateCurrentTeam)